/**
 * @param {string} template template string
 * @param {Object} params
 */
exports.template = function (template, params){
    // suppose:
    // template = 'abc{a}\\{b}';
    // params = { a: 1, b: 2 };

    if(!params){
        params = {};
    }
    
    // returns: 'abc1{b}'
    return ('' + template).replace(/\\?\{([^{}]+)\}/g, function(match, name){ // name -> match group 1
    
        // never substitute escaped braces `\\{}`
        // '\\{b}' -> '{b}'
        return match.charAt(0) === '\\' ? match.slice(1)
            :
                // '{a}' -> '1'
                ( params[name] !== null ? params[name] : '');
    });
};


/**
 * copy all properties in the supplier to the receiver
 * @param r {Object} receiver
 * @param s {Object} supplier
 * @param or {boolean=} whether override the existing property in the receiver
 * @param cl {(Array.<string>)=} copy list, an array of selected properties
 */
exports.mix = function(r, s, or, cl) {
    if (!s || !r){
        return r;
    }

    var i = 0, c, len;

    or = or || or === undefined;

    if (cl && (len = cl.length)) {
        for (; i < len; i++) {
            c = cl[i];
            if ( (c in s) && (or || !(c in r) ) ) {
                r[c] = s[c];
            }
        }
    } else {
        for (c in s) {
            if (or || !(c in r)) {
                r[c] = s[c];
            }
        }
    }
    return r;
};


// var obj = {a: {b: 2 }}
// obj, 'a.b' -> 2
exports.object_member_by_namespaces = function (obj, namespaces, default_value){
    var splitted = namespaces.split('.');
    var value = obj;

    splitted.some(function(ns) {
        if(ns in value){
            value = value[ns];
        }else{
            value = null;
            return true;
        }
    });

    return value || default_value;
};
